import express from 'express';
import bodyParser from 'body-parser';

const app = express();
import mongoose from 'mongoose';
// Mongoose
mongoose.connect('mongodb://localhost:27017/chatty',
    {useNewUrlParser: true});

// Various middleware, e.g. bodyParser, session, passport

// BodyParser enables POST requests to be inserted in req.body.
app.use(bodyParser.json({limit: '50mb'}));
app.use(bodyParser.urlencoded({ extended: true, limit: '50mb' }));

// CORS HEADER
// If you do assignments in the recommended order, you will find a problem with the CORS header.

const ALLOWED_WEBSITE = /https?:\/\/localhost(:\d*)?$/;
app.use((req, res, next) => {
    const origin = req.headers.origin as string;
    // It is OK to use headers submitted by the host in this case
    // since this protects the user's data from malicious websites.
        // If the browser is compromised, its really "not my daiji".
        // Besides, the only way you know which page this came from is... the browser.
    if (ALLOWED_WEBSITE.test(origin)) {
        res.header('Access-Control-Allow-Origin', origin);
        res.header('Access-Control-Allow-Headers', 'Origin, X-Requested-With, Content-Type, Accept');
    }
    next();
});


// App routing
import Messages from './routes/messages';

app.use('/messages', Messages);


// Serve the frontend
import path from 'path';
// Express static on the dist folder of your frontend, after `npm run build`
// https://www.npmjs.com/package/connect-history-api-fallback

// Launch server
import http from 'http';
const port = 3000;
const server = http.createServer(app);


// Websocket
import Primus from 'primus';
const primus = new Primus(server, {
    transformer: 'websockets',
});

// In practice, you would be getting this from a message service instead.
import { websocketHandler } from './routes/messages';
websocketHandler(primus);

app.get('/library.primus.js', (req, res) => {
    res.send(primus.library());
});

server.listen(port);

console.log(`App is running at http://localhost:${port} in ${app.get('env')} mode`);
