// There are two ways to implement the Model.
// - At the thread level
// - At the message level.

// Here i implement it at the Message level.

import { Schema, model, Document } from 'mongoose';

// ------------ Schema ------------
const MessageSchema = new Schema({
    to: { type: String, index: true, required: true },
    from: { type: String, index: true, required: true },
    msg: { type: String, required: true },
});

// Gets all messages involving a particular user.
MessageSchema.statics.findByUser = function(id: string) {
    return this.find({
        $or: [{
            to: id,
        }, {
            from: id,
        }],
    });
};

// ------------ Interface ------------

export interface Message extends Document {
    to: string;
    from: string;
    msg: string;
}

export interface MessageStatics {
    findByUser: (id: string) => Promise<Message[]>;
}

// ------------ Model ------------

// tslint:disable-next-line: variable-name
const _Message = model<Message>('Message', MessageSchema);

// This allows me to attach static type.
export const Message = _Message as (typeof _Message & MessageStatics);
