// tslint:disable: no-shadowed-variable
// tslint:disable: no-console
import express from 'express';


const route  = express();
export default route;

// YOUR ROUTES HERE.

import { UserId, IMessage, IUser } from '../../vue-typescript-solution/src/types';

route.get('/', (req, res) => {
    res.send(messages.lookup);
});

route.post('/new', (req, res) => {
    const { msg, to } = req.body;
    sendMessage(msg, to);
    res.send('OK');
});

// Your routes should be extra clean.
    // Routes are harder to test than functions.

// In practice, this should be segmented out into services/message.ts
// Routes should be left semi-strictly for routes.
// Generally if the implementation is self contained and only used here, you can leave it here.
// Otherwise you should move it.

// --------------- Normal server functions ---------------
const MY_USER_ID = '0'; // Change this when implementing user registration.
import { Message } from '../models/Message';

export interface IMessages {
    lookup: { [userId: string]: IMessage[] };
}


export const messages: IMessages = {
    lookup: {},
};

export function receiveMessage(msg: IMessage) {
    const userMsgs = messages.lookup[msg.sender] || [];
    userMsgs.push(msg);
    messages.lookup[msg.sender] = userMsgs;
    const newMsg = {
        to: MY_USER_ID,
        from: msg.sender,
        msg: msg.msg,
    };
    Message.create(newMsg)
    .then( () => {
        connected.forEach((spark) => {
            spark.write({
                channel: msg.sender,
                msg,
            });
        });
        console.log('Received', msg);
    });
}

export function sendMessage(msg: IMessage, to: UserId) {
    const userMsgs = messages.lookup[to] || [];
    userMsgs.push(msg);
    messages.lookup[to] = userMsgs;
    const { sender, msg: text } = msg;
    const newMsg = {
        to,
        from: sender,
        msg: text,
    };
    Message.create(newMsg)
    .then( () => {
        connected.forEach((spark) => {
            spark.write({
                channel: to,
                msg,
            });
        });
        console.log(`Send ${to}:`, msg);
    });
}

// -------------- WEBSOCKET --------------

import Primus from 'primus';

export function websocketHandler(primus: Primus) {
    primus.on('connection', (spark: Primus.Spark) => {
       handleConnection(spark);
    });
}

const connected: Primus.Spark[] = [];
function handleConnection(spark: Primus.Spark){
    connected.push(spark);
    spark.on('end', () => {
        _.pull(connected, spark); // Its O(n), but screw it :3
    });
}

// --------------- INITIAL ---------------

import data from '../data/friends.json'; // Did you know you can import data from JSON?

const friends: IUser[] = [];
const ids: string[] = [];
friends.push(...data);
ids.push(...data.map( (x) => x.id ));

console.log('finding');
Promise.all(ids.map( (id: string) => {
    return Message.findByUser(id)
        .then( (messages) => {
            console.log('found');
            return [id, messages];
        });
}))
.then( (results) => {
    console.log('Loaded messages', results);
});


// --------------- GENERATOR ---------------
// If you write something like this, refactor it into a new service.
// Then initialize it from this file.

// Configuration of message frequency, interval and shape.
const MIN_TIME = 500;
const MAX_TIME = 8000;
const INITIAL_MESSAGE_COUNT = 30;


import { LoremIpsum } from 'lorem-ipsum';

const lorem = new LoremIpsum({
    sentencesPerParagraph: {
      max: 4,
      min: 1,
    },
    wordsPerSentence: {
      max: 16,
      min: 4,
    },
});


// Fufufu, direct port xD
import _ from 'lodash';

// Randomly add more messages.
randomlyAddMessages();

function generateRandomInteger(max: number, min: number): number {
    return min + Math.floor( Math.random() * (max - min) );
}

function getRandomMessage() {
    const msg = lorem.generateParagraphs(1);
    const message: IMessage = {
        sender: _.sample(ids)!, // ! asserts that it will exist.
        msg,
    };
    return message;
}

function randomlyAddMessages() {
    const message = getRandomMessage();
    receiveMessage(message);

    const delay = generateRandomInteger(MAX_TIME, MIN_TIME);
    // Call this function again with a random delay.
    setTimeout(randomlyAddMessages, delay);
}
