import { UserId, IMessage, IUser } from '../types';
import { MY_USER_ID } from './Friends'; // In practice would be importing from User.
const NO_MSGS = 'No messages yet';
import Vue from 'vue';
import Axios from 'axios';


interface IMessages {
    lastUpdated?: Date;
    lookup: { [userId: string]: IMessage[] };
}


export const messages: IMessages = {
    lastUpdated: undefined,
    // This is the hack which allows force refreshing new properties
    lookup: {},
};

export function receiveMessage(msg: IMessage, to: UserId) {
    const userMsgs = messages.lookup[to] || [];
    userMsgs.push(msg);
    messages.lookup[to] = userMsgs;
    messages.lastUpdated = new Date();
}

export function sendMessage(msg: IMessage, to: UserId) {
    Axios.post('http://localhost:3000/messages/new', {
        msg,
        to,
    });
}

export function sendUserMessage(msg: string, to: UserId) {
    return sendMessage({
        sender: MY_USER_ID,
        msg,
    }, to);
}


export function getLastMessage(user: IUser) {
    const id = user.id;

    // NOTE: if you use messages.lookup[id] = [], you WILL NOT get reactive arrays.
    if (!messages.lookup[id]) {
        Vue.set(messages.lookup, id, []);
    }
    const userMessages = messages.lookup[id];
    if (!userMessages.length) {
        return NO_MSGS;
    }
    const lastMessage = userMessages[userMessages.length - 1];
    const prefix = lastMessage.sender === MY_USER_ID ? 'Me: ' : '';
    return `${prefix}${lastMessage.msg}`;
}


// ---------------- INITIAL DATA ----------------
Axios.get('http://localhost:3000/messages')
.then ( (resp) => {
    messages.lookup = resp.data;
    messages.lastUpdated = new Date();
});

declare const Primus: any;
const primus = new Primus('http://localhost:3000', {});

primus.on('data', (data: any) => {
    const {channel, msg} = data;
    receiveMessage(msg, channel);
});

